-- #####################################################################################
-- ## Skript zum Vortrag <Die Algorithmen hinter Xing, Facebooks und Navis...>        ##     
-- #####################################################################################
-- ## Dieses Skript legt die Projektdatenbank an. Es wird durch weitere Skripte       ##
-- ## ergnzt, die in ihrer durchnummerierten Reihenfolge abzuarbeiten sind. Die      ##
-- ## Skripte passen zu den Aufgabenstellungen aus der zugehrigen Powerpoint-        ##
-- ## Prsentation vom selben Autor.                                                  ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## nderungsnachweis:                                                              ##
-- ## 2019-06-29		Neuerstellung                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- auf <richtige> Datenbak wechseln
USE [Vortrag_Graph_Algorithmen]
GO


SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

-- ------------------------------------------------------------------------------------------------------------
-- Prozedur zur Suche (relationale Variante).                                                                --
-- ------------------------------------------------------------------------------------------------------------
-- Es werden zwei Parameter erwartet. Der erste reprsentiert die ID des Starts, der Zweite die ID des Ziels --
-- Zuerst sucht man alle verbindungen, die von der Start-ID ausgehen udn schreibt sie in die Tabelle         --
-- INITIALISIERUNG. In einer Schleife ergnzt man dann diese gefundenen Wege in der Tabelle ZWISCHENSPEICHER --
-- unter dem Hochzhlen der Sprunganzahl. Durch Umkopieren wird die Tabelle ZWISCHENSPEICHER dann zur        --
-- zur Tabelle INITIALISIERUNG. In der nchsten Runde greift der Algorithmus also auf den gefundene Weg zu   --
-- Als Abbruchbedingung reicht es, zu berprfen, ob der neue Pfad zu einem Element fhrt, welches schom im  --
-- aktuellen Weg vorkommt.                                                                                   --
-- ------------------------------------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [relational].[SucheVerbindungSchleife]
	  @PersonA AS BIGINT
	, @PersonB AS BIGINT
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @SchleifenZaehler AS INT
	DECLARE @AnzahlPersonen AS INT
	SET @SchleifenZaehler = 2
	SET @AnzahlPersonen = (SELECT count(DISTINCT PersonA) FROM [relational].[Bekanntschaft])

	-- In eine Zwischenspeichertabelle alle Verbindungen schreiben, die von der Start-ID (PersonA) ausgehen
	SELECT
		  [PersonA]																				AS [StartPerson]
		, [PersonB]																				AS [ZielPerson]
		, 1																						AS [Spruenge]
		, CONVERT(varchar(2), [Gesamtpfad]) + ' > ' + CONVERT(Varchar(2), [PersonB]	)			AS [Gesamtpfad]
	INTO [temp].[Initialisierung]
	FROM [relational].[Bekanntschaft]
	WHERE 1=1
		AND[PersonA] = @PersonA

	WHILE @SchleifenZaehler <= @AnzahlPersonen - 1
	BEGIN
		-- Unter Hochzhlen der Sprungzahl den Pfad um jede weitere Verbindung ergnzen
		SELECT
			  [bk].[PersonA]														AS [PersonA]
			, [bk].[PersonB]														AS [PersonB]
			, @Schleifenzaehler	+ 1													AS [Spruenge]
			, [ini].[Gesamtpfad] + ' > ' + CONVERT(VARCHAR(2), [bk].[PersonB])		AS [GesampPfad]
		INTO [temp].[PfadErgaenzung]
		FROM [relational].[Bekanntschaft] AS [bk]
			LEFT JOIN [temp].[Initialisierung] AS [ini] ON [bk].[PersonA] = [ini].[ZielPerson]
		WHERE 1=1
			AND [bk].[PersonA] IN (SELECT [ZielPerson] FROM [temp].[Initialisierung])
			AND [bk].[PersonA] <> @PersonB
			AND RIGHT([ini].[Gesamtpfad], 2) <> @PersonB
			AND [ini].[Spruenge] = @SchleifenZaehler - 1

		-- Umkopieren der bisherigen Ergebnise
		SELECT * 
		INTO [temp].[Zwischenspeicher]
		FROM [temp].[Initialisierung]

		-- Tabelle INITIALISIERUNG lschen, um sie gleich mit SELECT INTO wieder fllen zu knnen
		DROP TABLE [temp].[Initialisierung]

		-- Ringtausch, um auf der Initialsiierung des aktuellen Schrittes aufbauen zu knenn
		SELECT *
		INTO [temp].[Initialisierung]
		FROM 
		(
			SELECT 
				  [StartPerson]																	AS [StartPerson]
				, [ZielPerson]																	AS [ZielPerson]
				, [Spruenge]																	AS [Spruenge]
				, [Gesamtpfad]																	AS [Gesamtpfad]
			FROM [temp].[Zwischenspeicher]
			UNION
			SELECT
				  [PersonA]
				, [PersonB]
				, [Spruenge]
				, [GesampPfad]
			FROM [temp].[PfadErgaenzung]
		) AS Dummy

		DROP TABLE [temp].[PfadErgaenzung]
		DROP TABLE [temp].[Zwischenspeicher]

		SET  @SchleifenZaehler = @SchleifenZaehler + 1
	END

	-- finales Ergebnis ausgeben
	SELECT
		(SELECT LEN([ini].[Gesamtpfad]) - LEN(REPLACE([ini].[Gesamtpfad], '>', '')))				AS [Spruenge]
		, [Gesamtpfad]																	AS [Gesamtpfad]
	FROM [temp].[Initialisierung] AS [ini]
	WHERE 1=1
		AND RIGHT([ini].[Gesamtpfad], 2) = @PersonB
	ORDER BY [Spruenge] ASC

	DROP TABLE [temp].[Initialisierung]
END
GO
