-- #####################################################################################
-- ## Skript zum Vortrag <Die Algorithmen hinter Xing, Facebooks und Navis...>        ##     
-- #####################################################################################
-- ## Dieses Skript legt die Projektdatenbank an. Es wird durch weitere Skripte       ##
-- ## ergnzt, die in ihrer durchnummerierten Reihenfolge abzuarbeiten sind. Die      ##
-- ## Skripte passen zu den Aufgabenstellungen aus der zugehrigen Powerpoint-        ##
-- ## Prsentation vom selben Autor.                                                  ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## nderungsnachweis:                                                              ##
-- ## 2019-06-29		Neuerstellung                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- auf <richtige> Datenbak wechseln
USE [Vortrag_Graph_Algorithmen]
GO

-- ####################
-- ### Housekeeping ###
-- ####################

-- Tabelle KENNT im Schema GRAPH lschen, um sauber wieder aufsetzen zu knnen
DROP TABLE IF EXISTS [graph].[kennt]
GO

-- Tabelle PERSON im Schema GRAPH lschen, um sauber wieder aufsetzen zu knnen
DROP TABLE IF EXISTS [graph].[Person]
GO

-- Schema GRAPH lschen, um sauber wieder aufsetzen zu knnen
DROP SCHEMA IF EXISTS [graph]
GO

-- ########################
-- ### Aufbauarbeiten ####
-- ########################

-- Dieses Schema wird fr die Graphen-DB verwandt
CREATE SCHEMA [graph]
GO

-- Anlage der Tabelle PERSON im Schema GRAPH
-- zu beachsten ist die Anweisung AS NODE am Ende des Statements
CREATE TABLE [graph].[Person] (
	  ID INTEGER PRIMARY KEY
	, Vorname NVARCHAR(20)
	, Nachname NVARCHAR(20)
	) AS NODE;

-- Anlage der Tabelle KENNT im Schema GRAPH
-- zu beachsten ist die Anweisung AS EDGE am Ende des Statements
CREATE TABLE [graph].[kennt] (
	ID BIGINT PRIMARY KEY, 
	PersonA BIGINT, 
	PersonB BIGINT
	) AS EDGE;


-- Microsoft hat die SYS.TABLES-View angepasst und um entsprechende Kenner fr Graphen erweitert
SELECT 
	  schema_name(schema_id) AS [Schema]
	  , name
	, is_node
	, is_edge
FROM sys.tables
WHERE 1=1
	AND (name = 'kennt' OR name = 'Person');


-- Die Tabelle PERSON im Schema GRAPH wird mit Testdaten gefllt...
-- Die Syntax ist identisch zu "normalen" INSERTs
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 1, 'Max', 'Mustermann')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 2, 'Donalt', 'Dack')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 3, 'Mycki', 'Maus')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 4, 'Schneewiddchen', 'Grymm')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 5, 'Bibi', 'LongStromp')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 6, 'Maggy', 'Thtscher')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 7, 'Johnny', 'MacLaine')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 8, 'Rischar', 'Klaidrmann')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES ( 9, 'Brijitte', 'Bardoh')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES (10, 'Jiowanni', 'Trabbatohny')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES (11, 'Teigar', 'Wudz')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES (12, 'Bobbele', 'Bckr')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES (13, 'Pohl', 'MakHartnei')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES (14, 'Oddfried', 'Preusla')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES (15, 'Hlene', 'Fischa')
INSERT INTO [graph].[Person] ([ID], [Vorname], [Nachname]) VALUES (16, 'Ponzyus', 'Pilatous')

-- Einfgen von Testdaten in die Tabelle PERSON im Schema GRAPH
-- Die Syntax verlangt Subselects, um die Spalten $node_id und $from_id sinnvoll zu fllen
INSERT INTO [graph].[kennt] 
	VALUES (
		  (SELECT $node_id FROM [graph].[Person] WHERE ID = 1)
		, (SELECT $node_id FROM [graph].[Person] WHERE ID = 2)
		, 1					-- eigentlich berflssig, nur zu Demo-Zwecken !!! (Die IDs stehen auch in den versteckten Spalten!)
		, 1					-- eigentlich berflssig, nur zu Demo-Zwecken !!! (Die IDs stehen auch in den versteckten Spalten!)
		, 2);				-- eigentlich berflssig, nur zu Demo-Zwecken !!! (Die IDs stehen auch in den versteckten Spalten!)

INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 1), (SELECT $node_id FROM [graph].[Person] WHERE ID =12), 2, 1, 5);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 1), (SELECT $node_id FROM [graph].[Person] WHERE ID =12), 3, 1,12);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 1), (SELECT $node_id FROM [graph].[Person] WHERE ID =15), 4, 1,15);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 2), (SELECT $node_id FROM [graph].[Person] WHERE ID = 3), 5, 2, 3);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 2), (SELECT $node_id FROM [graph].[Person] WHERE ID = 5), 6, 2, 5);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 2), (SELECT $node_id FROM [graph].[Person] WHERE ID = 7), 7, 2, 7);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 2), (SELECT $node_id FROM [graph].[Person] WHERE ID =13), 8, 2,13);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 3), (SELECT $node_id FROM [graph].[Person] WHERE ID = 4), 9, 3, 4);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 3), (SELECT $node_id FROM [graph].[Person] WHERE ID = 6),10, 3, 6);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 3), (SELECT $node_id FROM [graph].[Person] WHERE ID =11),11, 3,11);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 4), (SELECT $node_id FROM [graph].[Person] WHERE ID = 2),12, 4, 2);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 4), (SELECT $node_id FROM [graph].[Person] WHERE ID = 6),13, 4, 6);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 4), (SELECT $node_id FROM [graph].[Person] WHERE ID = 8),14, 4, 8);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 5), (SELECT $node_id FROM [graph].[Person] WHERE ID = 3),15, 5, 3);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 5), (SELECT $node_id FROM [graph].[Person] WHERE ID =15),16, 5,15);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 5), (SELECT $node_id FROM [graph].[Person] WHERE ID =16),17, 5,16);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 6), (SELECT $node_id FROM [graph].[Person] WHERE ID =14),18, 6,14);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 7), (SELECT $node_id FROM [graph].[Person] WHERE ID =13),19, 7,13);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 8), (SELECT $node_id FROM [graph].[Person] WHERE ID =15),20, 8,15);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 9), (SELECT $node_id FROM [graph].[Person] WHERE ID = 7),21, 9, 7);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 9), (SELECT $node_id FROM [graph].[Person] WHERE ID =11),22, 9,11);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID = 9), (SELECT $node_id FROM [graph].[Person] WHERE ID =12),23, 9,12);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =10), (SELECT $node_id FROM [graph].[Person] WHERE ID = 2),24,10, 2);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =10), (SELECT $node_id FROM [graph].[Person] WHERE ID = 7),25,10, 7);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =10), (SELECT $node_id FROM [graph].[Person] WHERE ID =14),26,10,14);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =11), (SELECT $node_id FROM [graph].[Person] WHERE ID = 6),27,11, 6);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =11), (SELECT $node_id FROM [graph].[Person] WHERE ID =13),28,11,13);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =12), (SELECT $node_id FROM [graph].[Person] WHERE ID = 8),29,12, 8);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =13), (SELECT $node_id FROM [graph].[Person] WHERE ID =15),30,13,15);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =13), (SELECT $node_id FROM [graph].[Person] WHERE ID =16),31,13,16);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =14), (SELECT $node_id FROM [graph].[Person] WHERE ID = 3),32,14, 3);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =14), (SELECT $node_id FROM [graph].[Person] WHERE ID =15),33,14,15);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =15), (SELECT $node_id FROM [graph].[Person] WHERE ID =16),34,15,16);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =16), (SELECT $node_id FROM [graph].[Person] WHERE ID = 1),35,16, 1);
INSERT INTO [graph].[kennt] VALUES ((SELECT $node_id FROM [graph].[Person] WHERE ID =16), (SELECT $node_id FROM [graph].[Person] WHERE ID = 9),36,16, 9);

-- der SQL Server legt zustzliche Spalten an! Diese sind VERSTECKT
SELECT object_id, name, is_hidden, graph_type, graph_type_desc FROM	SYS.COLUMNS WHERE graph_type_desc IS NOT NULL

-- der SQL Server legt selber passende Indizes an
