-- #####################################################################################
-- ## Skript zum Vortrag <Die Algorithmen hinter Xing, Facebooks und Navis...>        ##     
-- #####################################################################################
-- ## Dieses Skript zeichnet die Positionsdaten der Flughfen in eine Landkarte       ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## nderungsnachweis:                                                              ##
-- ## 2019-06-29		Neuerstellung                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- A)


USE [Vortrag_Graph_Algorithmen]

SELECT COUNT(*) AS Anzahl
  FROM [relational].[Flugplan]



SELECT 
	  [Startflughafen]													AS [Startflughafen]
	, GEOGRAPHY::STPointFromText(
			'POINT(' 
			+ CONVERT(VARCHAR, [LaengengradStartflughafen]) 
			+ ' ' 
			+ CONVERT(VARCHAR, [BreitengradStartflughafen]) 
			+ ')'
		,4326).STBuffer(20000)											AS [Geo-Info]
FROM [relational].[Flugplan]
GROUP BY
	  [Startflughafen]
	, [LaengengradStartflughafen]
	, [BreitengradStartflughafen]
HAVING 1=1
	and isnumeric([LaengengradStartflughafen]) = 1
	and isnumeric([BreitengradStartflughafen]) = 1

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO



-- B)

-- Prozedur als CTE-Variante einspielen. Prozedur erwartet zwei Parameter (Start und Ziel)
CREATE OR ALTER PROCEDURE [relational].[SucheFlugCTE] 
	  @FlughafenA AS VARCHAR(3)
	, @FlughafenB AS VARCHAR(3)
AS
BEGIN
	SET NOCOUNT ON;

	;WITH cte 
	( 
			  [Startflughafen]
			, [Zielflughafen] 
			, [Sprung] 
			, [Flugroute] 
	) 
	AS ( 
			SELECT 
					  [v].[Startflughafen]											AS [Startflughafen]
					, [v].[Zielflughafen]											AS [Zielflughafen]
					, 0 															AS [Sprung]
					, CAST(' ' + CONVERT(VARCHAR, [v].[Startflughafen]) 
						+ ' ' + CONVERT(VARCHAR, [v].[Zielflughafen])
						+ ' ' AS VARCHAR(MAX))										AS [Flugroute]
			FROM [relational].[Flugplan] AS v 
			WHERE 1 = 1 
					AND [Startflughafen] = @FlughafenA 
        
			UNION ALL 
        
			SELECT 
					  [v].[Startflughafen]
					, [v].[Zielflughafen] 
					, [Sprung] + 1 
					, CAST([cte].[Startflughafen] + CONVERT(VARCHAR, [v].[Zielflughafen]) + ' ' AS VARCHAR(MAX)) 
			FROM [cte] 
			INNER JOIN [relational].[Flugplan] AS [v] ON [v].[Startflughafen] = [cte].[Zielflughafen]
			AND [Sprung] <= 1
			WHERE 1 = 1 
					AND [cte].[Startflughafen] NOT LIKE '% ' + CONVERT(VARCHAR, [v].[Zielflughafen]) + ' %' 
	) 
	SELECT 
			  [Sprung] + 1							AS [Sprung] 
			, LTRIM(RTRIM([Startflughafen]))		AS Flugroute 
	FROM [cte] 
	WHERE 1 = 1 
			  AND [Startflughafen] LIKE '% ' + CONVERT(VARCHAR, @FlughafenB) + ' ' 
	ORDER BY [Sprung] asc 
END
GO



-- C)
-- Dieses Skript luft auf dieser Hardware weit ber 1 Stunde! 

-- PIT --> ATL --> DET
DECLARE @RC int
DECLARE @FlughafenA VARCHAR(3)
DECLARE @FlughafenB VARCHAR(3)

SET @FlughafenA = 'PIT'
SET @FlughafenB = 'DET'

EXECUTE @RC = [relational].[SucheFlugCTE] 
   @FlughafenA
  ,@FlughafenB
GO


