-- #####################################################################################
-- ## Skript zum Vortrag <Die Algorithmen hinter Xing, Facebooks und Navis...>        ##     
-- #####################################################################################
-- ## Dieses Skript legt die Projektdatenbank an. Es wird durch weitere Skripte       ##
-- ## ergnzt, die in ihrer durchnummerierten Reihenfolge abzuarbeiten sind. Die      ##
-- ## Skripte passen zu den Aufgabenstellungen aus der zugehrigen Powerpoint-        ##
-- ## Prsentation vom selben Autor.                                                  ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## nderungsnachweis:                                                              ##
-- ## 2019-06-29		Neuerstellung                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- auf <richtige> Datenbak wechseln
USE [Vortrag_Graph_Algorithmen]
GO

-- ####################
-- ### Housekeeping ###
-- ####################

-- A)

-- Tabelle FLIEGT NACH im Schema GRAPH lschen, um sauber wieder aufsetzen zu knnen
DROP TABLE IF EXISTS [graph].[fliegt_nach]
GO

-- Tabelle FLUGHAFEN im Schema GRAPH lschen, um sauber wieder aufsetzen zu knnen
DROP TABLE IF EXISTS [graph].[Flughafen]
GO

-- Anlage der Tabelle Flughafen im Schema GRAPH
-- zu beachsten ist die Anweisung AS NODE am Ende des Statements
CREATE TABLE [graph].[Flughafen] (
	  [Startflughafen]			NVARCHAR(50)		PRIMARY KEY
	, [Breitengrad]				NVARCHAR(50)
	, [Laengengrad]				NVARCHAR(50)
	) AS NODE;

-- Anlage der Tabelle FLIEGT NACH im Schema GRAPH
-- zu beachsten ist die Anweisung AS EDGE am Ende des Statements
CREATE TABLE [graph].[fliegt_nach] (
	  [ID]						BIGINT			IDENTITY(1,1) NOT NULL
	, [Startflughafen]			NVARCHAR(500) 
	, [Zielflughafen]			NVARCHAR(500)
	, [Flugdatum]				DATE
	, [DistanzInMeilen]			INTEGER
	, [EinwohnerZahlStartOrt]	BIGINT
	, [EinwohnerzahlZielOrt]	BIGINT
	) AS EDGE;




-- B)

-- Microsoft hat die SYS.TABLES-View angepasst und um entsprechende Kenner fr Graphen erweitert
SELECT 
	  schema_name(schema_id) AS [Schema]
	  , name
	, is_node
	, is_edge
FROM sys.tables
WHERE 1=1
	AND (name = 'fliegt_nach' OR name = 'Flughafen');


-- C)

-- Die Tabelle FLUGHAFEN im Schema GRAPH wird mit Testdaten gefllt...
-- Die Syntax ist identisch zu "normalen" INSERTs
-- Die Daten werden aus der Tabelle FLUGHAFEN im Schema RELATIONAL ermittelt
INSERT INTO [graph].[Flughafen] (
				  [Startflughafen]
				, [Breitengrad]
				, [Laengengrad])
SELECT DISTINCT   [Startflughafen]
				, [BreitengradStartflughafen]
				, [LaengengradStartflughafen]
FROM [relational].[Flugplan]
WHERE 1=1
	AND [BreitengradStartflughafen] IS NOT NULL
	AND [LaengengradStartflughafen] IS NOT NULL
GO



-- D)

-- Hilfsview VFLUGVERBINDUNG erstellen
CREATE OR ALTER  VIEW [relational].[vFlugverbindung]
AS
SELECT DISTINCT 
		[Startflughafen]
      , [Zielflughafen]
      , [Flugdatum]
      , [DistanzInMeilen]
      , [EinwohnerZahlStartOrt]
      , [EinwohnerzahlZielOrt]
  FROM [Vortrag_Graph_Algorithmen].[relational].[Flugplan]
  WHERE 1=1
	AND [Startflughafen] IS NOT NULL
	AND [Zielflughafen] IS NOT NULL
	AND [Startflughafen] != [Zielflughafen]
 GO


 -- E)


-- Einfgen von Testdaten in die Tabelle FLIEGT NACH im Schema GRAPH
-- Die Syntax verlangt Subselects, um die Spalten $node_id und $from_id sinnvoll zu fllen
INSERT INTO [graph].[fliegt_nach] (
		  $from_id
		, $to_id
		, [Startflughafen]
		, [Zielflughafen]
		, [Flugdatum]
		, [DistanzInMeilen]
		, [EinwohnerZahlStartOrt]
		, [EinwohnerzahlZielOrt])
SELECT (SELECT TOP 1 $node_id FROM [graph].[Flughafen] [inn] WHERE [inn].[Startflughafen] = [Startflughafen]) 
		, (SELECT TOP 1 $node_id FROM [graph].[Flughafen] [inn] WHERE [inn].[Startflughafen] = [Zielflughafen])
		, [Startflughafen]
		, [Zielflughafen]
		, [Flugdatum]
		, [DistanzInMeilen]
		, [EinwohnerZahlStartOrt]
		, [EinwohnerzahlZielOrt]
  FROM [relational].[vFlugverbindung]
  WHERE 1=1
	AND (SELECT TOP 1 $node_id FROM [graph].[Flughafen] [inn] WHERE [inn].[Startflughafen] = [Startflughafen]) IS NOT NULL 
	AND (SELECT TOP 1 $node_id FROM [graph].[Flughafen] [inn] WHERE [inn].[Startflughafen] = [Zielflughafen]) IS NOT NULL 
GO



-- F)

INSERT INTO [graph].[fliegt_nach](
		  $from_id
		, $to_id
		, [Startflughafen]
		, [Zielflughafen]
		, [Flugdatum]
		, [DistanzInMeilen]
		, [EinwohnerZahlStartOrt]
		, [EinwohnerzahlZielOrt])
SELECT 
		  [j1].[Node1]
		, [j2].[Node2]
		, [vFv].[Startflughafen]
		, [vFv].[Zielflughafen]
		, [Flugdatum]
		, [DistanzInMeilen]
		, [EinwohnerZahlStartOrt]
		, [EinwohnerzahlZielOrt]
  FROM [relational].[vFlugverbindung] [vFv]
  INNER JOIN (SELECT $node_id AS [Node1], [Startflughafen] FROM [graph].[Flughafen]) [j1] 
	ON [j1].[Startflughafen] = [vFv].[Startflughafen]
  INNER JOIN (SELECT $node_id AS [Node2], [Startflughafen] FROM [graph].[Flughafen]) [j2] 
	ON [j2].[Startflughafen] = [vFv].[Startflughafen]
  WHERE 1=1


---- der SQL Server legt zustzliche Spalten an! Diese sind VERSTECKT
--SELECT object_id, name, is_hidden, graph_type, graph_type_desc FROM	SYS.COLUMNS WHERE graph_type_desc IS NOT NULL

---- der SQL Server legt selber passende Indizes an
